import logging
import os
import subprocess
from pathlib import Path

from .utils import highlight

SUPPORTED_SHELLS = {
    'bash': {
        'config_file': '.bashrc',
        'completion_directory': '.bash_completion.d',
    },
    'zsh': {
        'config_file': '.zshrc',
        'completion_directory': '.zsh/completions',
    },
    'fish': {
        'config_file': '.config/fish/config.fish',
        'completion_directory': '.config/fish/completions',
    }
}


def install_shell_completion():
    """Validate and update shell completion for the CLI"""
    shell = os.environ.get('SHELL', '').split('/')[-1]
    if not shell:
        logging.error("Could not determine shell type")
        return

    if shell not in SUPPORTED_SHELLS:
        logging.error(f"Unsupported shell: `{shell}`. "
                      f"Supported shells: {SUPPORTED_SHELLS}")
        return

    home = Path.home()
    completion_command = f"_P2P_COMPLETE={shell}_source p2p"
    completion_command = subprocess.run(
        completion_command,
        shell=True,
        check=True,
        capture_output=True,
        text=True
    ).stdout
    completion_directory = home / SUPPORTED_SHELLS[shell]['completion_directory']
    completion_directory.mkdir(parents=True, exist_ok=True)
    completion_file = completion_directory / '_p2p'
    completion_file.write_text(completion_command)
    run_completion_command = f"source {completion_file!s}"

    shell_config_file = home / SUPPORTED_SHELLS[shell]['config_file']
    shell_config_file.parent.mkdir(parents=True, exist_ok=True)

    with open(shell_config_file, 'a+') as f:
        f.seek(0)
        content = f.read()
        start_tag = "# >>> p2p shell completion >>>"
        end_tag = "# <<< p2p shell completion <<<"

        if run_completion_command not in content:
            f.write(f"\n{start_tag}")
            f.write(f"\n{run_completion_command}")
            f.write(f"\n{end_tag}")
            logging.info(f"Added completion script to {shell_config_file}")
        else:
            logging.info("Completion script already installed")

    print(f"Completion script installed to {shell_config_file}")
    print(f"Run `{highlight(run_completion_command)}` to enable completion "
          "in the current session")


import re
import sys
from dataclasses import dataclass, field

from openai import OpenAI

from .utils import get_logger

LOG = get_logger(__name__)

@dataclass
class Prompt:
    name: str
    content: str = field(default="")
    base_model: str = field(default=None)
    description: str = field(default="")
    enabled: bool = field(default=True)
    has_placeholder: bool = field(default=False)

    def __post_init__(self):
        self.validate()

    def validate(self):
        if not self.enabled:
            return

        missing_fields = []
        if self.content is None or self.content == "":
            missing_fields.append("content")
        if self.base_model is None:
            missing_fields.append("base_model")
        if len(missing_fields) > 0:
            missing_fields_str = ", ".join(missing_fields)
            LOG.error(f"Prompt '{self.name}' requires {missing_fields_str} fields")
            sys.exit(1)

        # Use regex to find all placeholders in the content
        allowed_placeholder = "pipe"
        format_vars = re.findall(r'{(.*?)}', self.content)
        for var_name in format_vars:
            if var_name != allowed_placeholder:
                LOG.error(f"Only '{allowed_placeholder}' placeholder is allowed, "
                          f"found: '{{{var_name}}}'")
                sys.exit(1)
            else:
                self.has_placeholder = True

    def run(self, pipe_input):

        if self.has_placeholder:
            if pipe_input is None or pipe_input == "":
                LOG.error(f"Empty input. Prompt '{self.name}' requires pipe input")
                sys.exit(1)
            else:
                message = self.content.format(pipe=pipe_input)
        else:
            message = self.content

        client = OpenAI()

        stream = client.chat.completions.create(
            model=self.base_model,
            messages=[{"role": "user", "content": message}],
            stream=True,
        )
        for chunk in stream:
            if chunk.choices[0].delta.content is not None:
                print(chunk.choices[0].delta.content, end="")
        
        client.close()


import os
import sys
import tomllib
from pathlib import Path

import click
from click import Command, Group

from .completion import install_shell_completion
from .prompt import Prompt

CONFIG_FILE = Path.home() / ".p2p" / "config.toml"


def load_prompts_from_toml():
    try:
        with open(CONFIG_FILE, "rb") as f:
            return tomllib.load(f)
    except FileNotFoundError:
        print(f"Config file not found: {CONFIG_FILE}")
        print("Run `p2p init` to create a new config file")
        return {}


class PromptCommand(Command, Prompt):
    def __init__(self, name, prompt_config):
        self.prompt_config = prompt_config

        Command.__init__(
            self,
            name=name,
            callback=self.run_prompt,
            params=[
                click.Argument(['pipe_input'], required=False)
            ],
            help=prompt_config.get('description', f'Execute {name} prompt'),
        )
        Prompt.__init__(self, name, **prompt_config)

    def get_short_help_str(self, limit=45):
        help_str = self.prompt_config.get(
            'description', f'Execute {self.name} prompt')
        help_str = "[PROMPT] " + help_str
        return help_str

    def run_prompt(self, pipe_input=None):
        if pipe_input is None and not sys.stdin.isatty():
            pipe_input = sys.stdin.read().strip()
        self.run(pipe_input)


class PromptManager(Group):
    """Manages prompt-related commands and configurations"""
    def __init__(self):
        super().__init__(name='prompt', help="Prompt management commands")
        self.add_command(self.list_prompts())

    def list_prompts(self):
        @click.command()
        @click.option('--long', '-l', is_flag=True,
                      help='Show long format including descriptions')
        def list(long):
            """List available prompts"""
            cli_instance = click.get_current_context().find_root().command
            prompts = {
                name: cmd for name, cmd in cli_instance.commands.items()
                if isinstance(cmd, PromptCommand)
            }
            if len(prompts) == 0:
                return

            max_name_length = max(len(name) for name, _ in prompts.items()) + 2

            if long:
                for name, cmd in sorted(prompts.items()):
                    padded_name = name.ljust(max_name_length)
                    click.echo(f"{click.style(padded_name, fg='green')}"
                               f"{cmd.get_short_help_str()}")
            else:
                terminal_width = os.get_terminal_size()[0]
                columns = max(1, terminal_width // max_name_length)
                
                line = []
                for idx, (name, _) in enumerate(sorted(prompts.items())):
                    line.append(name.ljust(max_name_length))
                    if len(line) == columns or idx == len(prompts) - 1:
                        click.echo(''.join(line).rstrip())
                        line = []

        return list


def init_command():
    @click.command()
    def init():
        """Initialize shell completion"""
        CONFIG_FILE.parent.mkdir(parents=True, exist_ok=True)
        if not CONFIG_FILE.exists():
            CONFIG_FILE.touch()
        install_shell_completion()
    return init


class CLI(Group):
    def __init__(self):
        super().__init__()
        self.prompts = load_prompts_from_toml()

        self.add_command(init_command())
        self.add_command(PromptManager())
        for prompt_name, prompt_config in self.prompts.items():
            if prompt_config.get('enabled', True):
                self.add_command(PromptCommand(prompt_name, prompt_config))

    def format_commands(self, ctx, formatter):
        """Override format_commands to customize the help output"""
        builtin_commands = []
        prompt_commands = []

        for subcommand in self.list_commands(ctx):
            cmd = self.get_command(ctx, subcommand)
            if cmd is None:
                continue
            if isinstance(cmd, PromptCommand):
                prompt_commands.append((subcommand, cmd))
            else:
                builtin_commands.append((subcommand, cmd))

        with formatter.section("Built-in Commands"):
            formatter.write_dl(
                [
                    (name, cmd.get_short_help_str())
                    for name, cmd in builtin_commands
                ]
            )
        
        if len(prompt_commands) > 0:
            with formatter.section("User Defined Prompts"):
                formatter.write_dl(
                    [
                        (name, cmd.get_short_help_str())
                        for name, cmd in prompt_commands
                    ]
                )

    def list_commands(self, ctx):
        """Return list of available commands"""
        return sorted(self.commands.keys())

    def get_command(self, ctx, cmd_name):
        """Get a specific command object"""
        cmd = self.commands.get(cmd_name)

        return cmd
    

cli = CLI()


if __name__ == '__main__':
    cli()



import logging

# ANSI color codes
COLORS = {
    'black': '\033[30m',
    'red': '\033[31m',
    'green': '\033[32m', 
    'yellow': '\033[33m',
    'blue': '\033[34m',
    'magenta': '\033[35m',
    'cyan': '\033[36m',
    'white': '\033[37m',
    'bright_black': '\033[90m',
    'bright_red': '\033[91m',
    'bright_green': '\033[92m',
    'bright_yellow': '\033[93m', 
    'bright_blue': '\033[94m',
    'bright_magenta': '\033[95m',
    'bright_cyan': '\033[96m',
    'bright_white': '\033[97m',
    'reset': '\033[0m'
}

def highlight(text: str, color: str = 'magenta') -> str:
    return f"{COLORS[color]}{text}{COLORS['reset']}"


# Custom formatter to add color to the log level
class ColoredFormatter(logging.Formatter):
    def format(self, record):
        levelname = record.levelname
        if levelname == 'ERROR':
            record.levelname = f"{COLORS['red']}{levelname}{COLORS['reset']}"
        elif levelname == 'WARNING':
            record.levelname = f"{COLORS['yellow']}{levelname}{COLORS['reset']}"
        return super().format(record)

def get_logger(name: str) -> logging.Logger:
    # Create a handler with the colored formatter
    handler = logging.StreamHandler()
    formatter = ColoredFormatter('%(levelname)s %(message)s')
    handler.setFormatter(formatter)

    # Get the logger and add the handler
    logger = logging.getLogger(name)
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG)  # Set the desired log level
    return logger
